/* ------------------------------------------------------------------------
 * This file is part of sciFLT ( Scilab Fuzzy Logic Toolbox )
 * Copyright (C) @YEARS@ Jaime Urzua Grez
 * mailto:jaime_urzua@yahoo.com
 * 
 * 2011 Holger Nahrstaedt
 * ------------------------------------------------------------------------
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * ------------------------------------------------------------------------
 * CHANGES:
 * 2004-11-04 Delete some variable initialization, clean code
 *            Use more optimized memory subroutines
 * 2006-08-28 Use SCILAB memory allocation
 * ------------------------------------------------------------------------
 */

/*  
 * ------------------------------------------------------------------------
 * DESCRIPTION:
 * This routine is the computational part of:
 * (1) return an internal representation of a fls structure 
 * [mid1,mid2,mew,mrule,mdomo,mpari,mparo]=flsencode(fls)
 * (2) evaluate a fls
 * [y,tmp2]=evalfls(x,fls)
 * 
 * TYPE      DESCRIPTION
 * --------- -----------------------------------------
 * (double ) mdomo=matrix [nouputs,2] -> domain of outputs variables -> used in Mamdani case
 * (integer) mrule=matrix [nrules,ninputs+noutputs+1] -> fls.rule(:,1:(ninputs+noutputs+1)) but with member function id
 * (double ) mpari=matrix [4,ninputs*nrules] -> input member functions parameters
 * (double ) mparo=matrix [q,noutputs*nrules] q=4 if is mamdani or q=ninputs+1 if is takagi-sugeno
 * (double ) mew=matrix[nrules,1] -> fls.rule(:,$+1)
 * (integer) mid1=matrix [7,1]
 * (double ) mid2=matrix [3,1]
 * 
 * mid1(1)= { 0=m , 1=ts } -> fls.type
 * mid1(2)= { 0=dubois , 1=yager, 2=dsum , 3=esum  , 4=asum , 5=max } -> fls.SNorm
 * mid1(3)= { 0=dubois , 1=yager, 2=dprod, 3=eprod , 4=aprod, 5=min } -> fls.TNorm
 * mid1(4)= { 0=one, 1=yager, 2=sugeno } -> fls.Comp
 * mid1(5)= { 0=min, 1=prod } -> fls.ImpMethod
 * mid1(6)= { 0=max, 1=sum, 2=probor } -> fls.AggMethod
 * mid1(7)= { 0=centroide, 1=bisector, 2=mom, 3=som, 4=lom, 5=waver, 6=wsum } -> fls.defuzzMethod
 * 
 * mid2(1) = fls.SNormPar;
 * mid2(2) = fls.TNormPar;
 * mid2(3) = fls.CompPar;
 * ------------------------------------------------------------------------ 
 */

#include <string.h>
#include "stack-c.h" 
#include "api_scilab.h"
#include "Scierror.h"
#include "MALLOC.h"
// #include "cvstr.h"
// #include "core_math.h"

//#include "sciFLT.h"

#define sign(a) ((a) < 0 ? -1 : 1)
#define SCIFLT_MAXNPEV 1001
#define FLT_USESCIALLOC
/* -------------------------------------------------------------------------------------------------------- */
/* EXTERNAL                                                                                                 */
/* -------------------------------------------------------------------------------------------------------- */

/* ---------------------------------------------------------------- */
/* INTERFACE TO flsencode                                           */
/* ---------------------------------------------------------------- */
int inter_flsencode(char * fname)

{
	CheckRhs(1,1);
	CheckLhs(7,7);
	/* flsencode routine have all */
	flsencode(0);
	return (0);
}

/* ---------------------------------------------------------------- */
/* INTERFACE TO evalfls                                             */
/* ---------------------------------------------------------------- */
int inter_evalfls(char * fname)

{
	CheckRhs(2,3);
	CheckLhs(1,2);
	/* flsencode routine have all */
	flsencode(1);
	return (0);
}



