/* -----------------------------------------------------------------------
 * SCICOS FLS COMPUTATION BLOCK
 * -----------------------------------------------------------------------
 * This file is part of sciFLT ( Scilab Fuzzy Logic Toolbox )
 * Copyright (C) @YEARS@ Jaime Urzua Grez
 * mailto:jaime_urzua@yahoo.com
 * 
 * 2011 Holger Nahrstaedt
 * -----------------------------------------------------------------------
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * -----------------------------------------------------------------------
 * CHANGES :
 * 2004-11-08 Change error messages, Add flag detection -> The routine can
 *            be better!
 * -----------------------------------------------------------------------
 */

#include "stack-c.h" 
#include "api_scilab.h"
#include "Scierror.h"
#include "MALLOC.h"
//#include "sciFLT.h"
#include <machine.h>
#include <scicos_block4.h>


// int C2F(sfls)(flag, nevprt, t, xd, x, nx, z, nz, tvec, ntvec, rpar, nrpar, ipar, nipar, u, nu, y, ny)
//      int *flag, *nevprt,*nx,*nz,*nrpar, *ipar, *nipar,*ntvec,*nu,*ny;
//      double *t, *xd, *x, *z, *tvec, *rpar, *u, *y;
// {
void sfls(scicos_block *block,int flag) 
{
	int i,tofree[5],ierr,idx,one=1,ncolmparo;
	int *npev,*mrule,*mid1;
	double *mdomo,*mpari,*mparo,*mew,*mid2,*tmp1,*tmp2,tmp3,*tmp4;
	int maxnpev=1001,freeme;
	
	int ninputs,noutputs,nrules;
	int* ipar = GetIparPtrs(block);
	double* rpar = GetRparPtrs(block);
	int nu = GetNin(block);
	double * u = GetInPortPtrs(block,1);
	double * y = GetOutPortPtrs(block,1);
	freeme=0;

	/* ----------------------------------------------------------- */
	/* COMPUTATIONAL PART                                          */
	/* ----------------------------------------------------------- */
	if ( flag==1 ) {
		
		/* Check if the user choose a fls structure */
		if ( ipar[0]!=1 ) { sciFLT_doerror(2000); flag=-999; goto goout; }

		/* Get information */
		ninputs=ipar[1];
		noutputs=ipar[2];
		nrules=ipar[3];

		/* Chek if are somethig to evaluate */
		if (ninputs*noutputs*nrules==0) { sciFLT_doerror(2001); flag=-999; goto goout; }
		
		/* Check the input size */
		if (ninputs!=nu) { sciFLT_doerror(2002); flag=-999; goto goout; }		
		
		/* Get the rest of integer parameters */
		idx=4;
		mrule=&ipar[idx];
		idx+=nrules*(ninputs+noutputs+1);
		mid1=&ipar[idx];

		/* Get the ral values */
		idx=0;
		mdomo=&rpar[0];
		idx+=noutputs*2;
		mpari=&rpar[idx];
		idx+=4*ninputs*nrules;
		mparo=&rpar[idx];
		if (mid1[0]==0) {
			ncolmparo=4;
		} else {
			ncolmparo=ninputs+1;
		}
		idx+=ncolmparo*noutputs*nrules;
		mew=&rpar[idx];
		idx+=nrules;
		mid2=&rpar[idx];

		/* Intiliaze */
		freeme=1;
		for (i=0;i<5;i++) { tofree[i]=0; }
		#ifdef FLT_USESCIALLOC
		tmp1=(double*)MALLOC(ninputs*sizeof(double));
		#else
		tmp1=(double*)malloc(ninputs*sizeof(double));
		#endif
		if (tmp1==NULL) { sciFLT_doerror(999); flag=-999; goto goout; }
		tofree[0]=1;

		#ifdef FLT_USESCIALLOC
		tmp2=(double*)MALLOC(nrules*sizeof(double));
		#else
		tmp2=(double*)malloc(nrules*sizeof(double));
		#endif
		if (tmp2==NULL) { sciFLT_doerror(999); flag=-999; goto goout; }
		tofree[1]=1;

		if ( mid1[0]==0 ) {
			#ifdef FLT_USESCIALLOC
			npev=(int*)MALLOC(noutputs*sizeof(int));
			#else
			npev=(int*)malloc(noutputs*sizeof(int));
			#endif
			if (npev==NULL) { sciFLT_doerror(999); flag=-999; goto goout; }
			tofree[4]=1;
			for (i=0;i<noutputs;i++) { npev[i]=maxnpev; }
			#ifdef FLT_USESCIALLOC
			tmp4=(double*)MALLOC(maxnpev*3*sizeof(double));
			#else
			tmp4=(double*)malloc(maxnpev*3*sizeof(double));
			#endif
			if (tmp4==NULL) { sciFLT_doerror(999); flag=-999; goto goout; }
			tofree[3]=1;
		}
	
		/* Make the real work */
		C2F(flsengine)(u,mid1,mid2,mew,mrule,mdomo,mpari,mparo,&ncolmparo,npev,&ninputs,&noutputs,&nrules,&one,tmp1,tmp2,&tmp3,tmp4,&maxnpev,y,&ierr);
		if (ierr!=0) { flag=-999; }
	}
			
goout:
	if (freeme==1) {
		#ifdef FLT_USESCIALLOC
		if (tofree[0]==1 ) { FREE(tmp1); }
		if (tofree[1]==1 ) { FREE(tmp2); }
		if (tofree[3]==1 ) { FREE(tmp4); }
		if (tofree[4]==1 ) { FREE(npev); }
		#else
		if (tofree[0]==1 ) { free(tmp1); }
		if (tofree[1]==1 ) { free(tmp2); }
		if (tofree[3]==1 ) { free(tmp4); }
		if (tofree[4]==1 ) { free(npev); }
		#endif
	}

	return 0;
}

